local addonInfo, privateVars = ...

if privateVars.ui == nil then privateVars.ui = {} end

function privateVars.ui.Button(buttonType)
	
	local lastmount, lastpet;
	local name = "nkZooButton." .. buttonType
	local mouseDown;
	local mountFlag, lastMountTime = false, nil
	
	local button = EnKai.uiCreateFrame ('nkActionButton', name, privateVars.uiElements.context)
	button:SetScale(nkZooSettings.scale / 100)
	button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nkZooSettings.buttonDef[buttonType].x, nkZooSettings.buttonDef[buttonType].y)
	button:SetSecureMode('restricted')
	
	if nkZooSettings.locked == true then
    button:SetDragable(false)
  else
    button:SetDragable(true)
  end
	
  function button:update()
	
	  if buttonType == 'mount' then
      button:setMountMacro()
    else
      button:setPetMacro()
    end

	end
	
	function button:resize()
    button:SetScale(nkZooSettings.scale / 100)
  end
  	
  function button:setMountMacro()
  	local mount, justOneMount = nil, false
    
    while mount == nil or (mount == lastmount and justOneMount == false) do
      mount, justOneMount = button:getRandomMount()
      if mount == nil then return end
    end 
    
    if mount == nil then return end
    
    local data = nkItemBase.Query.byKey(mount, nil, "mount")
    
    if data == nil then 
      lastmount = mount
      return
    end
    
    local mountDesc = data[string.lower(EnKai.tools.lang.getLanguageShort())]
    local macro = string.format ('use %s', mountDesc)
    button:SetTexture("Rift", string.format('Data/\\UI\\item_icons\\%s.dds', data.icon))
    button:SetMacro ( macro)  
    
    lastmount = mount
	end
	
	function button:SetMounted(flag, updatePanel)
	  mountFlag = flag
	  
	  if flag == true then lastMountTime = Inspect.Time.Real() end
	  
	  if updatePanel == true and privateVars.uiElements.panel ~= nil and privateVars.uiElements.panel:GetMountButton() ~= nil then
      privateVars.uiElements.panel:GetMountButton():SetMounted(flag, false)
    end
	  
	end
	
  function button:getRandomMount()
  
    local mountTable = {}
  
    for k, v in pairs(nkZooSettings.mounts) do
      if v == true then table.insert(mountTable, k) end
    end
    
    if #mountTable == 0 then 
      return nil
    elseif #mountTable == 1 then
      return mountTable[1], true
    end
  
    local randomNo = math.random(#mountTable)
    return mountTable[randomNo] 
  
  end
  
  function button:setPetMacro()

    local pet, justOnePet = nil, false
    
    while pet == nil or (pet == lastpet and justOnePet == false) do
      pet, justOnePet = button:getRandomPet()
      if pet == nil then return end
    end 
      
    local data = nkItemBase.Query.byKey(pet, nil, "pets")
    if data == nil then 
      lastpet = pet
      return 
    end
    
    local petDesc = data[string.lower(EnKai.tools.lang.getLanguageShort())]
    
    local macro = string.format ('use %s', petDesc)
    button:SetTexture("Rift", string.format('Data/\\UI\\item_icons\\%s.dds', data.icon))
    button:SetMacro (macro)  
    
    lastpet = pet
    
  end
  
  function button:getRandomPet()

    local petTable = {}
  
    for k, v in pairs(nkZooSettings.pets) do
      if v == true then table.insert(petTable, k) end
    end
    
    if #petTable == 0 then 
      return nil 
    elseif #petTable == 1 then
      return petTable[1], true
    end
  
    local randomNo = math.random(#petTable)
    return petTable[randomNo] 
  
  end
  
  function button:ShowHelp()

    local anchorElement = button
  
    local from, to = "BOTTOM", "TOP"
    
    if nkZooSettings.buttonDef[buttonType].y < 100 then from = "TOP" end
    
    if nkZooSettings.buttonDef[buttonType].x > -250 then 
      from = from .. 'RIGHT'
      to = to .. 'LEFT'
    else
      from = from .. 'LEFT'
      to = to .. 'RIGHT'
    end
      
    if privateVars.uiElements.tooltip == nil then privateVars.uiElements.tooltip = EnKai.uiCreateFrame('nkItemTooltip', 'nkZooTooltip', button) end
    
    if privateVars.uiElements.tooltip.lastFrom ~= nil then
      privateVars.uiElements.tooltip:ClearPoint(privateVars.uiElements.tooltip.lastFrom)  
    end 
    
    privateVars.uiElements.tooltip:SetPoint(from, anchorElement, to)
    privateVars.uiElements.tooltip.lastFrom = from
    
    local itemData
    
    if buttonType == 'mount' then
      if lastmount ~= nil then itemData = nkItemBase.Query.byKey(lastmount, nil, "mount") end      
    else
      if lastpet ~= nil then itemData = nkItemBase.Query.byKey(lastpet, nil, "pets") end
    end
    
    if itemData ~= nil then    
      privateVars.uiElements.tooltip:SetItem(itemData.akey, itemData)
      privateVars.uiElements.tooltip:SetVisible(true)
    end
    
  end
  
  -- EVENTS
  
  button:EventAttach(Event.UI.Input.Mouse.Left.Click, function (self)
    if Inspect.System.Secure() == false then
      if privateVars.uiElements.tooltip ~= nil then privateVars.uiElements.tooltip:SetVisible(false) end
      if buttonType  == 'mount' then
        if mountFlag ~= true and (lastMountTime == nil or Inspect.Time.Real() - lastMountTime > 2) then
          button:SetMounted(true, true)
          button:setMountMacro()
          lastMountTime = Inspect.Time.Real();
        else
          button:SetMounted(false, true)          
        end
      else
        button:setPetMacro()        
      end
    end
  end, name .. ".LeftClick")
  
  button:EventAttach(Event.UI.Input.Mouse.Middle.Click, function (self)
    if Inspect.System.Secure() == false then
      if privateVars.uiElements.configUI == nil then privateVars.uiElements.configUI = privateVars.ui.Config() else privateVars.uiElements.configUI:SetVisible(true) end 
    end
  end, name .. ".MiddleClick")
  
  Command.Event.Attach(EnKai.events[name].Moved, function (_, xpos, ypos)
    nkZooSettings.buttonDef[buttonType].x = xpos
    nkZooSettings.buttonDef[buttonType].y = ypos
  end, name .. '.Moved')

  button:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
    if Inspect.System.Secure() == false then button:ShowHelp(buttonType) end
  end, name .. ".MouseIn")
  
  button:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
    if Inspect.System.Secure() == false then
      if privateVars.uiElements.tooltip ~= nil then privateVars.uiElements.tooltip:SetVisible(false) end
    end
   end, name .. ".MouseOut")
      
	return button

end