local addonInfo, privateVars = ...

privateVars.layout = {
  elementColor = { 0.153, 0.314, 0.490, 1 },
  labelColor = { 1, 1, 1, 1 }
}

if privateVars.ui == nil then privateVars.ui = {} end

function privateVars.ui.Config(settings)

  local name = "privateVars.ui.Config"

  local config = EnKai.uiCreateFrame('nkWindowMetro', name, privateVars.uiElements.context)
  config:SetLayer(1)
  config:SetWidth(650)
  config:SetHeight(600)
  config:SetTitle(addonInfo.name)
  config:SetPoint("CENTER", UIParent, "CENTER")
  
  local EnKaiLogo = EnKai.uiCreateFrame("nkTexture", name .. ".EnKaiLogo", config)
  EnKaiLogo:SetTextureAsync(EnKai.art.GetThemeLogo()[1],EnKai.art.GetThemeLogo()[2])
  EnKaiLogo:SetPoint("BOTTOMLEFT", config:GetContent(), "BOTTOMLEFT", 10, -5)
  EnKaiLogo:SetWidth(125)
  EnKaiLogo:SetHeight(33)
  
  local closeButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".closeButton", config:GetContent())
  
  closeButton:SetPoint("BOTTOMRIGHT", config:GetContent(), "BOTTOMRIGHT", -10, -10)
  closeButton:SetText(privateVars.langTexts.btClose)
  closeButton:SetIcon("nkZoo", "gfx/iconClose.png")
  closeButton:SetScale(.8)
  closeButton:SetLayer(9)
  
  Command.Event.Attach(EnKai.events[name .. ".closeButton"].Clicked, function (_, newValue)
    Command.Event.Detach(Event.Tooltip, nil, "nkZoo.Event.Tooltip")
    privateVars.scan = false
    config:SetVisible(false) 
  end, name .. ".closeButton.Clicked")
 
  local tabPane = EnKai.uiCreateFrame("nkTabPaneMetro", name .. ".tabPane", config:GetContent())
  tabPane:SetBorder(false)
  tabPane:SetPoint("TOPLEFT", config:GetContent(), "TOPLEFT", 10, 10)
  tabPane:SetPoint("BOTTOMRIGHT", config:GetContent(), "BOTTOMRIGHT", -10, -50)
  tabPane:SetLayer(1)
  
  local paneTabCategories = privateVars.ui.configTabConfig(name .. ".configTabConfig", tabPane)
  local paneTabSettings = privateVars.ui.configTabSettings(name .. ".configTabSettings", tabPane)
  
  tabPane:AddPane( { label = privateVars.langTexts.tabHeader1, frame = paneTabCategories, initFunc = function() paneTabCategories:build() end}, false)
  tabPane:AddPane( { label = privateVars.langTexts.tabHeader2, frame = paneTabSettings, initFunc = function() paneTabSettings:build() end}, true)
  
  local versionText = EnKai.uiCreateFrame("nkText", name .. ".versionText", config)
  versionText:SetFontSize(11)
  versionText:SetText(string.format(privateVars.langTexts.txtVersion, addonInfo.toc.Version))
  versionText:SetPoint("BOTTOMRIGHT", tabPane, "BOTTOMRIGHT", -5, -5)
  versionText:SetLayer(98)
    
  function config:updateUI ()
    if privateVars.uiElements.mountButton ~= nil then 
      privateVars.uiElements.mountButton:SetVisible(nkZooSettings.buttonDef['mount'].show) 
      privateVars.uiElements.mountButton:resize()
    end
    
    if privateVars.uiElements.petButton ~= nil then 
      privateVars.uiElements.petButton:SetVisible(nkZooSettings.buttonDef['pet'].show) 
      privateVars.uiElements.petButton:resize()
    end
  end
  	
  return config

end

function privateVars.ui.configTabConfig (name, parent)

  local frame = EnKai.uiCreateFrame("nkFrame", name, parent)
  local grid, slider, typeSelect
  
  local gridRows = 19  

  function frame:build()
   
    typeSelect = EnKai.uiCreateFrame("nkCombobox", name .. '.typeSelect', frame)
    typeSelect:SetPoint("TOPLEFT", frame, "TOPLEFT")
    typeSelect:SetWidth(300)
    typeSelect:SetLabelWidth(100)
    typeSelect:SetText( privateVars.langTexts.typeLabel)
    typeSelect:SetSelection(privateVars.langTexts.typeSelection)
    typeSelect:SetSelectedValue('mount', false)
    typeSelect:SetLayer(9)
    typeSelect:SetLayer(9)
    
    Command.Event.Attach(EnKai.events[name .. '.typeSelect'].ComboChanged, function (_, newValue)
      frame:PopulateGrid()       
    end, name .. ".typeSelect.ComboChanged")
       
    -- grid definition
    
    grid = EnKai.uiCreateFrame("nkGrid", name .. '.grid', frame)
    
    grid:SetHeaderHeight(25)
    grid:SetPoint("TOPLEFT", typeSelect, "BOTTOMLEFT", 0, 10)
    grid:SetTransparentHeader()
    grid:SetSortable(true)
    grid:SetLayer(2)
    
    local cols = {  {header = privateVars.langTexts.gridHeaderActive, width = 50, texture = true, textureType = 'nkZoo'}, 
                    {align = 'left', header = privateVars.langTexts.gridHeaderMount, width = 340 },
                    {header = privateVars.langTexts.gridHeaderSpeed, width = 200}
                 }
    
    grid:SetVisible(false)  
    grid:Layout (cols, gridRows)  
                  
    Command.Event.Attach(EnKai.events[name .. '.grid'].GridFinished, function ()
      frame:PopulateGrid() 
      grid:SetVisible(true)
    end, name .. '.grid.GridFinished')
    
    Command.Event.Attach(EnKai.events[name .. '.grid'].WheelForward, function (_, rowPos)
      slider:AdjustValue (rowPos)
    end, name .. '.grid.Grid.WheelForward')
    
    Command.Event.Attach(EnKai.events[name .. '.grid'].WheelBack, function (_, rowPos)
      slider:AdjustValue (rowPos)
    end, name .. '.grid.Grid.WheelBack')
    
    Command.Event.Attach(EnKai.events[name .. '.grid'].MouseOver, function (_, rowPos)
    
      local realRow = grid:GetRowPos() + rowPos - 1
      if grid:GetKey(realRow) == nil then return end
      
      privateVars.internal.showTooltip(grid:GetKey(realRow))
      self.tooltipRow = realRow
    end, name .. '.grid.Grid.MouseOver')
    
    Command.Event.Attach(EnKai.events[name .. '.grid'].MouseOut, function (_, rowPos)
      if rowPos == nil or self.tooltipRow == (grid:GetRowPos() + rowPos - 1) then
        if privateVars.ui.configTooltip ~= nil then privateVars.ui.configTooltip:SetVisible(false) end
        self.tooltipRow = nil
      end
    end, name .. '.grid.Grid.MouseOut')
    
    Command.Event.Attach(EnKai.events[name .. '.grid'].LeftClick, function (_, row)
      
      if grid:GetKey(row) == nil then return end
      
      local itemType = typeSelect:GetSelectedValue()
    
      local values = nkZooSettings.mounts 
      if itemType ~= 'mount' then values =  nkZooSettings.pets end  
      
      if values[grid:GetKey(row)] == true then
        values[grid:GetKey(row)] = false
      else
        values[grid:GetKey(row)] = true
      end
      
      frame:PopulateGrid(row)

    end, name .. '.grid.LeftClick')       
    
    -- grid slider
    
    slider = EnKai.uiCreateFrame("nkScrollbox", name .. ".slider", frame) 
    
    slider:SetPoint("TOPLEFT", grid, "TOPRIGHT", 0, 25)
    slider:SetHeight(grid:GetHeight() - 25)
    slider:SetRange(1, 1)
    slider:SetVisible(false)
    slider:SetLayer(2)
    
    Command.Event.Attach(EnKai.events[name .. ".slider"].ScrollboxChanged, function ()    
      grid:SetRowPos(math.floor(slider:GetValue('value')), true)
    end, name .. '.slider.ScrollboxChanged')
    
    local scanButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".scanButton", frame)
  
    scanButton:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, -5)
    scanButton:SetText(privateVars.langTexts.btScan)
    scanButton:SetIcon("nkZoo", "gfx/iconScan.png")
    scanButton:SetWidth(200)
    scanButton:SetScale(.8)
    scanButton:SetLayer(9)
    
    Command.Event.Attach(EnKai.events[name .. ".scanButton"].MouseIn, function (_, newValue)
      privateVars.internal.scanTooltip(scanButton)
    end, name .. ".scanButton.MouseIn")
    
    Command.Event.Attach(EnKai.events[name .. ".scanButton"].MouseOut, function (_, newValue)
      if privateVars.ui.scanTooltip  ~= nil then privateVars.ui.scanTooltip:SetVisible(false) end
    end, name .. ".scanButton.MouseOut")
    
    Command.Event.Attach(EnKai.events[name .. ".scanButton"].Clicked, function (_, newValue)
    
      if privateVars.scan == true then
        Command.Event.Detach(Event.Tooltip, nil, "nkZoo.Event.Tooltip")
        privateVars.scan = false
        scanButton:SetColor(privateVars.layout.elementColor[1], privateVars.layout.elementColor[2], privateVars.layout.elementColor[3])
        frame:PopulateGrid()
      else
        Command.Event.Attach(Event.Tooltip, privateVars.internal.tooltip, "nkZoo.Event.Tooltip")
        privateVars.scan = true
        scanButton:SetColor(0.078, 0.118, 0.306)
      end
       
    end, name .. ".scanButton.Clicked")
    
    local toggleButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".toggleButton", frame)
    
    toggleButton:SetPoint("CENTERLEFT", scanButton, "CENTERRIGHT", 10, 0)
    toggleButton:SetText(privateVars.langTexts.btToggle)
    toggleButton:SetIcon("nkZoo", "gfx/iconToggle.png")
    toggleButton:SetWidth(200)
    toggleButton:SetScale(.8)
    toggleButton:SetLayer(9)
    
    Command.Event.Attach(EnKai.events[name .. ".toggleButton"].Clicked, function (_, newValue)
    
      for k, v in pairs(nkZooSettings.mounts) do
        nkZooSettings.mounts[k] = false
      end
      
      for k, v in pairs(nkZooSettings.pets) do
        nkZooSettings.pets[k] = false
      end
       
      frame:PopulateGrid() 
       
    end, name .. ".scanButton.Clicked")
    
  end
  
  function frame:PopulateGrid(rowPos)
    
    local itemType = typeSelect:GetSelectedValue()
    
    local gridValues = {} 
    
    if itemType == 'mount' then
      
      local sysMounts = nkItemBase.Query.allByType ('mount', nil, nil, false, true)
      
      for k, v in pairs (sysMounts) do
        local active = ''
        local owned = ''
        if nkZooSettings.mounts[k] == true then active = "gfx/statusGreen.png" else active = "gfx/statusRed.png" end
        
        local desc = v[string.lower(EnKai.tools.lang.getLanguageShort())]
        local speedText = nil
        
        if v.speed == nil then
          speedText = ''
        elseif v.speed == 0 then
          speedText = privateVars.langTexts.txtSpeedVariable
        else
          if v.fixed == true then
            speedText = string.format ('%d%%', v.speed)
          else
            speedText = string.format (privateVars.langTexts.txtSpeedMinOrFastest, v.speed)
          end
        end
        
        table.insert (gridValues, {active, { key = k, value = desc, color = nkItemBase.qualityColor[tonumber(v.q)]}, speedText})
      end
    else
      local sysPets = nkItemBase.Query.allByType ('pets', nil, nil, false, true)
    
      for k, v in pairs (sysPets) do
        local active = ''
        if nkZooSettings.pets[k] == true then active = "gfx/statusGreen.png" else active = "gfx/statusRed.png" end
        
        local desc = v[string.lower(EnKai.tools.lang.getLanguageShort())]
        table.insert (gridValues, {active, { key = k, value = desc, color = nkItemBase.qualityColor[tonumber(v.q)]}, ''})
            
      end
    end
    
    table.sort (gridValues, function (v1, v2) return v1[2].value < v2[2].value end )
  
    grid:SetCellValues(gridValues)
    if rowPos == nil then grid:SetRowPos(1, false) end
      
    if (#gridValues-gridRows) > 0 then    
      slider:SetRange (1, #gridValues-gridRows+1)
       if rowPos == nil then slider:AdjustValue(1) end
      slider:SetVisible(true)     
    else
      slider:SetVisible(false)
    end
  end
  
  return frame

end

function privateVars.ui.configTabSettings (name, parent)

  local frame = EnKai.uiCreateFrame("nkFrame", name, parent)
  local pluginCheckbox, lockButtonsCheckbox, scaleSlider, mountButtonCheckbox, petButtonCheckbox
  
  function frame:build()
  
    pluginCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. '.pluginCheckbox', frame)  
    pluginCheckbox:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 10)
    pluginCheckbox:SetLabelWidth(150)
    pluginCheckbox:SetText(privateVars.langTexts.usePlugin)
    pluginCheckbox:SetChecked(nkZooSettings.usePlugin)
    pluginCheckbox:SetLayer(2)
    
    Command.Event.Attach(EnKai.events[name .. '.pluginCheckbox'].CheckboxChanged, function (_, newValue)   
      nkZooSettings.usePlugin = newValue
      
      if privateVars.uiElements.panel ~= nil then
        nkPanel.api.togglePanel('nkZoo', newValue)
      elseif newValue == true then
        local items = { { label = privateVars.langTexts.buttonMenuConfig, callBack = function () if privateVars.uiElements.configUI == nil then privateVars.uiElements.configUI = privateVars.ui.Config() else privateVars.uiElements.configUI:SetVisible(true) end end} }
        privateVars.uiElements.panel = privateVars.internal.nkPanelPlugin(items)
        nkPanel.api.registerPlugin('nkZoo', privateVars.uiElements.panel)   
      end
      
    end, name .. '.pluginCheckbox.CheckboxChanged')
  
    lockButtonsCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. '.lockButtonsCheckbox', frame)  
    lockButtonsCheckbox:SetPoint("TOPLEFT", pluginCheckbox, "BOTTOMLEFT", 0, 10)
    lockButtonsCheckbox:SetLabelWidth(150)
    lockButtonsCheckbox:SetText(privateVars.langTexts.buttonsLocked)
    lockButtonsCheckbox:SetChecked(nkZooSettings.locked)
    lockButtonsCheckbox:SetLayer(2)
    
    Command.Event.Attach(EnKai.events[name .. '.lockButtonsCheckbox'].CheckboxChanged, function (_, newValue)   
      nkZooSettings.locked = newValue
      if newValue == true then
        privateVars.uiElements.mountButton:SetDragable(false)
        privateVars.uiElements.petButton:SetDragable(false)
      else
        privateVars.uiElements.mountButton:SetDragable(true)
        privateVars.uiElements.petButton:SetDragable(true)
      end
    end, name .. '.lockButtonsCheckbox.CheckboxChanged')
  
    scaleSlider = EnKai.uiCreateFrame("nkSlider", name .. ".scaleSlider", frame)
    scaleSlider:SetRange(20, 200)
    scaleSlider:SetMidValue(100)
    scaleSlider:SetWidth(250)
    scaleSlider:SetLabelWidth(150)
    scaleSlider:SetText(privateVars.langTexts.slScale) 
    scaleSlider:SetPoint("TOPLEFT", lockButtonsCheckbox, "BOTTOMLEFT", 0, 10)
    scaleSlider:AdjustValue(nkZooSettings.scale)
    
    Command.Event.Attach(EnKai.events[scaleSlider:GetName()].SliderChanged, function (_, newValue)
      nkZooSettings.scale = newValue
      --scaleSlider:SetText(string.format(privateVars.langTexts.slScale, nkZooSettings.scale)) 
      privateVars.uiElements.configUI:updateUI()
    end, scaleSlider:GetName() .. '.SliderChanged')
    
    mountButtonCheckbox  = EnKai.uiCreateFrame("nkCheckbox", name .. ".mountButtonCheckbox", frame)
    mountButtonCheckbox:SetChecked(nkZooSettings.buttonDef['mount'].show)
    mountButtonCheckbox:SetLabelWidth(150)
    mountButtonCheckbox:SetText(privateVars.langTexts.cbMountButton)
    mountButtonCheckbox:SetPoint("TOPLEFT", scaleSlider, "BOTTOMLEFT",0 ,10)
    
    Command.Event.Attach(EnKai.events[mountButtonCheckbox:GetName()].CheckboxChanged, function (_, newValue)
      nkZooSettings.buttonDef['mount'].show = newValue
      privateVars.uiElements.configUI:updateUI()
    end, mountButtonCheckbox:GetName() .. '.CheckboxChanged')
    
    petButtonCheckbox  = EnKai.uiCreateFrame("nkCheckbox", name .. ".petButtonCheckbox", frame)
    petButtonCheckbox:SetChecked(nkZooSettings.buttonDef['pet'].show)
    petButtonCheckbox:SetLabelWidth(150)
    petButtonCheckbox:SetText(privateVars.langTexts.cbPetButton)
    petButtonCheckbox:SetPoint("TOPLEFT", mountButtonCheckbox, "BOTTOMLEFT",0 ,10)
    
    Command.Event.Attach(EnKai.events[petButtonCheckbox:GetName()].CheckboxChanged, function (_, newValue)
      nkZooSettings.buttonDef['pet'].show = newValue
      privateVars.uiElements.configUI:updateUI()
    end, petButtonCheckbox:GetName() .. '.CheckboxChanged')
  end

  return frame
  
end

function privateVars.internal.showTooltip(key)
	
	if key == nil then return end
	
	if privateVars.ui.configTooltip == nil then privateVars.ui.configTooltip = EnKai.uiCreateFrame('nkItemTooltip', 'nkZooConfigTooltip', privateVars.uiElements.tooltipContext) end

	local itemData = nkItemBase.Query.byKey(key, nil, "mount,pets")	
	if itemData == nil then return end
	
	privateVars.ui.configTooltip:SetItem(itemData.akey, itemData)
	
	local mouseData = Inspect.Mouse()
	
	privateVars.ui.configTooltip:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouseData.x, mouseData.y)
	privateVars.ui.configTooltip:SetLayer(99)
	privateVars.ui.configTooltip:SetVisible(true)
					
end

function privateVars.internal.scanTooltip(parent)

  if privateVars.ui.scanTooltip == nil then 
    privateVars.ui.scanTooltip = EnKai.uiCreateFrame('nkTooltip', 'nkZooScanTooltip', privateVars.uiElements.tooltipContext)   
    privateVars.ui.scanTooltip:SetTitle(privateVars.langTexts.txtHintTitle)
    privateVars.ui.scanTooltip:SetLines({{text = privateVars.langTexts.txtHintBody, wordwrap=true, minWidth = 200}})
    privateVars.ui.scanTooltip:SetPoint("TOPLEFT", parent, "BOTTOMRIGHT", -10, -10)
    privateVars.ui.scanTooltip:SetLayer(99)    
  end
  
  privateVars.ui.scanTooltip:SetVisible(true)

end