local addonInfo, privateVars = ...

privateVars.internal = {}

privateVars.uiElements = {}
privateVars.uiElements.context = UI.CreateContext("nkZoo")
privateVars.uiElements.context:SetSecureMode('restricted')
privateVars.uiElements.context:SetStrata("dialog")

privateVars.uiElements.tooltipContext = UI.CreateContext("nkZooTooltip")
privateVars.uiElements.tooltipContext:SetStrata("topmost")

function privateVars.internal.main(_, addon)

	if addon == addonInfo.identifier then
	
		local items = { { label = privateVars.langTexts.buttonMenuConfig, callBack = function () if privateVars.uiElements.configUI == nil then privateVars.uiElements.configUI = privateVars.ui.Config() else privateVars.uiElements.configUI:SetVisible(true) end end} }
				
		if nkPanel ~= nil and nkZooSettings.usePlugin == true then
      privateVars.uiElements.panel = privateVars.internal.nkPanelPlugin(items)
      nkPanel.api.registerPlugin('nkZoo', privateVars.uiElements.panel)          
    else      
      EnKai.manager.init('nkZoo', items, nil)      
    end
		
		math.randomseed(os.time())
		
		if nkZooSettings.buttonDef.mount.show == true and privateVars.uiElements.mountButton == nil then privateVars.uiElements.mountButton = privateVars.ui.Button('mount') end
		if nkZooSettings.buttonDef.pet.show == true and privateVars.uiElements.petButton == nil then privateVars.uiElements.petButton = privateVars.ui.Button('pet') end
		
		if privateVars.uiElements.mountButton ~= nil then privateVars.uiElements.mountButton:update() end
		if privateVars.uiElements.petButton ~= nil then privateVars.uiElements.petButton:update() end
		
		table.insert(Command.Slash.Register("nkZoo"), {privateVars.internal.commandHandler, "nkZoo", "config"})
		
		print (string.format(privateVars.langTexts.msgLoaded, addonInfo.toc.Version))
				
	end
	
end

function privateVars.internal.commandHandler(commandline)

	if privateVars.uiElements.configUI == nil then privateVars.uiElements.configUI = privateVars.ui.Config() else privateVars.uiElements.configUI:Show(true) end
	
end

function privateVars.internal.settingsHandler(_, addon)

  if addon == addonInfo.identifier then
	
  	local settings = {
  	  usePlugin = true,
  		selectedMount = 1,						
  		buttonDef = {	pet 	= { x = 50, y = 250, show = true, texture = 'gfx/petButton.png' },
  						      mount	= { x = 100, y = 250, show = true, texture = 'gfx/mountButton.png'}},
  		mountButton = true,
  		petButton = true,	
  		scale = 100,
  		locked = false,
  		mounts = {},
  		pets = {}
  	}
  	
  	if nkZooSettings == nil then
  		nkZooSettings = settings
  	elseif nkZooSettings.buttonDef.pet.x < 0 or nkZooSettings.buttonDef.pet.y < 0 then
  	  -- do full reset for V3.0.0
       nkZooSettings = settings
    else
  		for k, v in pairs (settings) do if nkZooSettings[k] == nil then nkZooSettings[k] = v end end
  	end
  	
  	if nkZooSettings.buttonDef.mount.texture ~= 'gfx/mountButton.png' then nkZooSettings.buttonDef.mount.texture = 'gfx/mountButton.png' end
  	if nkZooSettings.buttonDef.pet.texture ~= 'gfx/petButton.png' then nkZooSettings.buttonDef.pet.texture = 'gfx/petButton.png' end
  	
  	if nkZooSettings.usePlugin == nil then nkZooSettings.usePlugin = true end
  	
  end
	
end

------------------- EVENTS -------------------

function privateVars.internal.tooltip (_, type, info)

  if privateVars.scan ~= true then return end
  
  local itemCategory, abilityId, itemType

  if type == "item" or type == "itemtype" then
    local temp = Inspect.Item.Detail(info)
    
    if temp == nil then return end
    
    if temp.category == 'misc mount' then
      local dbDetails = nkItemBase.Query.byKey(temp.type, true, "mount", false)
      if dbDetails ~= nil then
        nkZooSettings.mounts[dbDetails.key] = true
      end
    elseif temp.category == 'misc pet' then
      local dbDetails = nkItemBase.Query.byKey(temp.type, true, "pets", false)
      if dbDetails ~= nil then
        nkZooSettings.pets[dbDetails.key] = true
      end
    end
  end
  
end

function privateVars.internal.castbar(_, info)
  
  if privateVars.playerID == nil then
      local playerDetails = Inspect.Unit.Detail('player')
      if playerDetails == nil then return end
      privateVars.playerID = playerDetails.id
  end
  
  if privateVars.uiElements.mountButton == nil or privateVars.uiElements.panel == nil or privateVars.uiElements.panel:GetMountButton() == nil then return end
  
  for k, v in pairs(info) do
    if k == privateVars.playerID then
      local castDetails = Inspect.Unit.Castbar(k)
      if castDetails ~= nil and castDetails.abilityNew ~= nil then
        local ability = Inspect.Ability.New.Detail(castDetails.abilityNew)
        if ability.name == 'Mounting' then
          if privateVars.uiElements.mountButton ~= nil then
            privateVars.uiElements.mountButton:SetMounted(true, true)
          elseif privateVars.uiElements.panel ~= nil and privateVars.uiElements.panel:GetMountButton() ~= nil then
            privateVars.uiElements.panel:GetMountButton():SetMounted(true, true)
          end          
        end
      end
    end
  end
  
end

function privateVars.internal.death(_, info)

  if privateVars.playerID == nil then
      local playerDetails = Inspect.Unit.Detail('player')
      if playerDetails == nil then return end
      privateVars.playerID = playerDetails.id
  end
  
  if privateVars.uiElements.mountButton == nil or privateVars.uiElements.panel == nil or privateVars.uiElements.panel:GetMountButton() == nil then return end
  
  if info.target == privateVars.playerID then
    if privateVars.uiElements.mountButton ~= nil then
      privateVars.uiElements.mountButton:SetMounted(false, true)
    elseif privateVars.uiElements.panel ~= nil and privateVars.uiElements.panel:GetMountButton() ~= nil then
      privateVars.uiElements.panel:GetMountButton():SetMounted(false, true)
    end       
  end

end

Command.Event.Attach(Event.Addon.Load.End, privateVars.internal.main, "nkZoo.Addon.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, privateVars.internal.settingsHandler, "nkZoo.SavedVariables.Load.End")

Command.Event.Attach(Event.Unit.Castbar, privateVars.internal.castbar, "nkZoo.Unit.Castbar")
Command.Event.Attach(Event.Combat.Death, privateVars.internal.death, "nkZoo.Combat.Death")