local addonInfo, privateVars = ...

function privateVars.internal.nkPanelPlugin ()

	local plugin = nkPanel.api.plugin('nkZoo plugin', false, true)
	plugin:SetSecureMode('restricted')

	--local customTitle = nkDevTools.ui(plugin)
	
	local customTitle = UI.CreateFrame('Frame', 'nkZoo.plugin.title', plugin)
	customTitle:SetSecureMode('restricted')
	customTitle:SetWidth(61)
	customTitle:SetHeight(plugin:GetHeight())
	customTitle:SetMouseMasking('limited')
	
	local thumb = UI.CreateFrame('Texture', 'nkZoo.plugin.title.thumb', customTitle)
	thumb:SetHeight(16)
	thumb:SetWidth(16) 
	thumb:SetTextureAsync('nkZoo', 'gfx/nkPanelIcon.png')
	thumb:SetPoint("CENTERLEFT", customTitle, "CENTERLEFT")
	thumb:SetMouseMasking('limited')
	
	local mountButton = privateVars.internal.nkPanelPluginButton ('mount', customTitle)
	mountButton:SetPoint("CENTERLEFT", thumb, "CENTERRIGHT", 5, 0)
	
	local petButton = privateVars.internal.nkPanelPluginButton ('pet', customTitle)
	petButton:SetPoint("CENTERLEFT", mountButton, "CENTERRIGHT", 2, 0)
		
	plugin:SetCustomTitle(customTitle)
	
	local body = plugin:CreateBody()
	body:SetSecureMode('restricted')
	
	local menu = EnKai.uiCreateFrame("nkMenu", 'nkZoo.plugin.menu', body)
		
	menu:SetFontSize(13)
	menu:SetWidth(120)
	menu:SetBackgroundColor(0, 0, 0, 0)
	menu:SetLabelColor(1, 1, 1, 1)
	menu:SetBorderColor(0, 0, 0, 0)
	menu:SetPoint("TOPLEFT", body, "TOPLEFT", 2, 2)
	menu:SetSecureMode('restricted')
	
	menu:AddEntry({ closeOnClick = true, label = privateVars.langTexts.buttonMenuConfig, callBack = function () if privateVars.uiElements.configUI == nil then privateVars.uiElements.configUI = privateVars.ui.Config() else privateVars.uiElements.configUI:SetVisible(true) end end })
	
	body:SetHeight(menu:GetHeight()+4)
	body:SetWidth(menu:GetWidth()+4)
	
	function plugin:GetMountButton() return mountButton end
	
	function plugin:GetMenu() return menu end
  function plugin:GetBody() return body end
	
	return plugin

end

function privateVars.internal.nkPanelPluginButton(buttonType, parent)
  
  local lastmount, lastpet;
  local name = "nkZooButton.panel." .. buttonType
  local mouseDown;
  local mountFlag, lastMountTime = false, nil
  
  local buttonSize = 16
  
  local button = EnKai.uiCreateFrame ('nkActionButton', name, parent)
  button:SetScale(.4)
  button:SetSecureMode('restricted')
  
  function button:update()
  
    if buttonType == 'mount' then
      button:setMountMacro()
    else
      button:setPetMacro()
    end

  end
  
  function button:SetMounted(flag, updateButton)
    mountFlag = flag
    if flag == true then lastMountTime = Inspect.Time.Real() end
    
    if updateButton == true and privateVars.uiElements.mountButton ~= nil then privateVars.uiElements.mountButton:SetMounted(flag, false) end
  end
  
  function button:setMountMacro()
    local mount, justOneMount = nil, false
    
    while mount == nil or (mount == lastmount and justOneMount == false) do
      mount, justOneMount = button:getRandomMount()
      if mount == nil then return end
    end 
    
    if mount == nil then return end
    
    local data = nkItemBase.Query.byKey(mount, nil, "mount")
    
    if data == nil then 
      lastmount = mount
      return
    end
    
    local mountDesc = data[string.lower(EnKai.tools.lang.getLanguageShort())]
    local macro = string.format ('use %s', mountDesc)
    button:SetTexture("Rift", string.format('Data/\\UI\\item_icons\\%s.dds', data.icon))
    button:SetMacro ( macro)  
    
    lastmount = mount
  end
  
  function button:getRandomMount()
  
    local mountTable = {}
  
    for k, v in pairs(nkZooSettings.mounts) do
      if v == true then table.insert(mountTable, k) end
    end
    
    if #mountTable == 0 then 
      return nil
    elseif #mountTable == 1 then
      return mountTable[1], true
    end
  
    local randomNo = math.random(#mountTable)
    return mountTable[randomNo] 
  
  end
  
  function button:setPetMacro()

    local pet, justOnePet = nil, false
    
    while pet == nil or (pet == lastpet and justOnePet == false) do
      pet, justOnePet = button:getRandomPet()
      if pet == nil then return end
    end 
      
    local data = nkItemBase.Query.byKey(pet, nil, "pets")
    if data == nil then 
      lastpet = pet
      return 
    end
    
    local petDesc = data[string.lower(EnKai.tools.lang.getLanguageShort())]
    
    local macro = string.format ('use %s', petDesc)
    button:SetTexture("Rift", string.format('Data/\\UI\\item_icons\\%s.dds', data.icon))
    button:SetMacro (macro)  
    
    lastpet = pet
    
  end
  
  function button:getRandomPet()

    local petTable = {}
  
    for k, v in pairs(nkZooSettings.pets) do
      if v == true then table.insert(petTable, k) end
    end
    
    if #petTable == 0 then 
      return nil 
    elseif #petTable == 1 then
      return petTable[1], true
    end
  
    local randomNo = math.random(#petTable)
    return petTable[randomNo] 
  
  end
  
  -- EVENTS
  
  button:EventAttach(Event.UI.Input.Mouse.Left.Click, function (self)
    if Inspect.System.Secure() == false then
      if buttonType  == 'mount' then
        if mountFlag ~= true and (lastMountTime == nil or Inspect.Time.Real() - lastMountTime > 2) then
          button:SetMounted(true, true)
          button:setMountMacro()
        else
          button:SetMounted(false, true)          
        end
      else
        button:setPetMacro()        
      end
    end
  end, name .. ".LeftClick")
  
  button:update()
  
  return button

end